@@ -1,4 +1,4 @@
-/* config.c - MemTest-86  Version 3.2
+/* config.c - MemTest-86  Version 3.3
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady
@@ -16,8 +16,9 @@ extern int bail;
 extern struct tseq tseq[];
 extern short e820_nr;
 extern char memsz_mode;
+void performance();
 
-char save[2][POP_H][POP_W];
+char save[2][SPD_H][SPD_W];
 
 void get_config()
 {
@@ -35,8 +36,8 @@ void get_config()
 		cprint(POP_Y+6,  POP_X+6, "(4) Error Summary");
 		cprint(POP_Y+7,  POP_X+6, "(5) Error Report Mode");
 		cprint(POP_Y+8,  POP_X+6, "(6) ECC Mode"); 
-		cprint(POP_Y+9,  POP_X+6, "(7) Restart");
-		cprint(POP_Y+10, POP_X+6, "(8) Refresh Screen");
+		cprint(POP_Y+9,  POP_X+6, "(7) Restart w/Defaults");
+		cprint(POP_Y+10, POP_X+6, "(8) Redraw Screen");
 		cprint(POP_Y+11, POP_X+6, "(9) Adv. Options");
 		cprint(POP_Y+12,POP_X+6,"(0) Continue");
 
@@ -52,7 +53,7 @@ void get_config()
 			cprint(POP_Y+4, POP_X+6, "(2) Skip Current Test");
 			cprint(POP_Y+5, POP_X+6, "(3) Select Test");
 			cprint(POP_Y+6, POP_X+6, "(4) Select Bit Fade Test");
-			cprint(POP_Y+7, POP_X+6, "(0) Continue");
+			cprint(POP_Y+7, POP_X+6, "(0) Cancel");
 			if (v->testsel < 0) {
 				cprint(POP_Y+3, POP_X+5, ">");
 			} else {
@@ -124,7 +125,7 @@ void get_config()
 			cprint(POP_Y+3, POP_X+6, "(1) Set Lower Limit");
 			cprint(POP_Y+4, POP_X+6, "(2) Set Upper Limit");
 			cprint(POP_Y+5, POP_X+6, "(3) Test All Memory");
-			cprint(POP_Y+6, POP_X+6, "(0) Continue");
+			cprint(POP_Y+6, POP_X+6, "(0) Cancel");
 			wait_keyup();
 			while (!sflag) {
 				switch(get_key()) {
@@ -196,7 +197,7 @@ void get_config()
 				cprint(POP_Y+2+memsz_mode, POP_X+5, ">");
 			} else {
 				cprint(POP_Y+4, POP_X+6, "(3) Probe");
-				cprint(POP_Y+5, POP_X+6, "(0) Continue");
+				cprint(POP_Y+5, POP_X+6, "(0) Cancel");
 				if (memsz_mode == SZ_MODE_BIOS) {
 					cprint(POP_Y+3, POP_X+5, ">");
 				} else {
@@ -210,40 +211,16 @@ void get_config()
 					memsz_mode = SZ_MODE_BIOS;
 					wait_keyup();
 					restart();
-/*
-					mem_size();
-					v->test = 0;
-					v->pass = 0;
-					v->total_ticks = 0;
-					bail++;
-					sflag++;
-*/
 					break;
 				case 3:
 					memsz_mode = SZ_MODE_BIOS_RES;
 					wait_keyup();
 					restart();
-/*
-					mem_size();
-					v->test = 0;
-					v->pass = 0;
-					v->total_ticks = 0;
-					bail++;
-					sflag++;
-*/
 					break;
 				case 4:
 					memsz_mode = SZ_MODE_PROBE;
 					wait_keyup();
 					restart();
-/*
-					mem_size();
-					v->test = 0;
-					v->pass = 0;
-					v->total_ticks = 0;
-					bail++;
-					sflag++;
-*/
 					break;
 				case 11:
 				case 57:
@@ -275,7 +252,7 @@ void get_config()
 			cprint(POP_Y+3, POP_X+6, "(1) Individual Errors");
 			cprint(POP_Y+4, POP_X+6, "(2) BadRAM Patterns");
 			cprint(POP_Y+5, POP_X+6, "(3) Error Counts Only");
-			cprint(POP_Y+6, POP_X+6, "(0) Continue");
+			cprint(POP_Y+6, POP_X+6, "(0) Cancel");
 			cprint(POP_Y+3+v->printmode, POP_X+5, ">");
 			wait_keyup();
 			while (!sflag) {
@@ -313,7 +290,7 @@ void get_config()
 			cprint(POP_Y+3, POP_X+6, "(1) Recommended");
 			cprint(POP_Y+4, POP_X+6, "(2) On");
 			cprint(POP_Y+5, POP_X+6, "(3) Off");
-			cprint(POP_Y+6, POP_X+6, "(0) Continue");
+			cprint(POP_Y+6, POP_X+6, "(0) Cancel");
 			wait_keyup();
 			while(!sflag) {
 				switch(get_key()) {
@@ -347,7 +324,33 @@ void get_config()
 			flag++;
 			break;
                 case 10:
-			get_menu();
+			/* 9 - Advanced Options */
+			popclear();
+			cprint(POP_Y+1, POP_X+2, "Advanced Options:");
+			cprint(POP_Y+3, POP_X+6, "(1) Display SPD Info");
+			cprint(POP_Y+4, POP_X+6, "(2) Modify Memory Timing");
+			cprint(POP_Y+5, POP_X+6, "(0) Cancel");
+			wait_keyup();
+			while (!sflag) {
+				switch(get_key()) {
+				case 2:
+					popdown();
+					show_spd();
+					popup();
+					sflag++;
+					break;
+				case 3:
+					get_menu();
+					sflag++;
+					break;
+				case 11:
+				case 57:
+					/* 0/CR - Continue */
+					sflag++;
+					break;
+				}
+			}
+			popclear();
 			break;
 		case 11:
 		case 57:
@@ -366,6 +369,56 @@ void get_config()
         }
 }
 
+void performance()
+{ 
+	extern int l1_cache, l2_cache;
+	ulong speed;
+	int i;
+
+	popclear();
+	cprint(POP_Y+1, POP_X+1, "             Read   Write    Copy");
+	cprint(POP_Y+3, POP_X+1, "L1 Cache:");
+	speed=memspeed((ulong)mapping(0x100), (l1_cache/4)*1024, 50, MS_READ);
+	dprint(POP_Y+3, POP_X+10, speed, 6, 0);
+	speed=memspeed((ulong)mapping(0x100), (l1_cache/4)*1024, 50, MS_WRITE);
+	dprint(POP_Y+3, POP_X+17, speed, 6, 0);
+	speed=memspeed((ulong)mapping(0x100), (l1_cache/4)*1024, 50, MS_COPY);
+	dprint(POP_Y+3, POP_X+24, speed, 6, 0);
+
+	if (l2_cache < l1_cache) {
+		i = l1_cache / 4 + l2_cache / 4;
+	} else {
+		i = l1_cache;
+	}
+	cprint(POP_Y+5, POP_X+1, "L2 Cache:");
+	speed=memspeed((ulong)mapping(0x100), i*1024, 50, MS_READ);
+	dprint(POP_Y+5, POP_X+10, speed, 6, 0);
+	speed=memspeed((ulong)mapping(0x100), i*1024, 50, MS_WRITE);
+	dprint(POP_Y+5, POP_X+17, speed, 6, 0);
+	speed=memspeed((ulong)mapping(0x100), i*1024, 50, MS_COPY);
+	dprint(POP_Y+5, POP_X+24, speed, 6, 0);
+
+	/* Determine memory speed.  To find the memory spped we use */
+        /* A block size that is 5x the sum of the L1 and L2 caches */
+        i = (l2_cache + l1_cache) * 5;
+
+        /* Make sure that we have enough memory to do the test */
+        if ((1 + (i * 2)) > (v->plim_upper << 2)) {
+                i = ((v->plim_upper <<2) - 1) / 2;
+        }
+	cprint(POP_Y+7, POP_X+1, "Memory:");
+	speed=memspeed((ulong)mapping(0x100), i*1024, 50, MS_READ);
+	dprint(POP_Y+7, POP_X+10, speed, 6, 0);
+	speed=memspeed((ulong)mapping(0x100), i*1024, 50, MS_WRITE);
+	dprint(POP_Y+7, POP_X+17, speed, 6, 0);
+	speed=memspeed((ulong)mapping(0x100), i*1024, 50, MS_COPY);
+	dprint(POP_Y+7, POP_X+24, speed, 6, 0);
+
+	wait_keyup();
+	while (get_key() == 0);
+	popclear();
+}
+	
 
 void popup()
 {
