@@ -1,4 +1,4 @@
-/* init.c - MemTest-86  Version 3.2
+/* init.c - MemTest-86  Version 3.3
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady
@@ -18,13 +18,14 @@
 extern short memsz_mode;
 extern short firmware;
 
+int l1_cache=0, l2_cache=0;
 struct cpu_ident cpu_id;
 ulong st_low, st_high;
 ulong end_low, end_high;
 ulong cal_low, cal_high;
 ulong extclock;
 
-static ulong memspeed(ulong src, ulong len, int iter);
+ulong memspeed(ulong src, ulong len, int iter, int type);
 static void cpu_type(void);
 static void cacheable(void);
 static int cpuspeed(void);
@@ -51,7 +52,7 @@ static void display_init(void)
 	for(i=0, pp=(char *)(SCREEN_ADR+1); i<TITLE_WIDTH; i++, pp+=2) {
 		*pp = 0x47;
 	}
-	cprint(0, 0, "      Memtest-86 v3.2       ");
+	cprint(0, 0, "      Memtest-86 v3.3       ");
 
 	/* Do reverse video for the bottom display line */
 	for(i=0, pp=(char *)(SCREEN_ADR+1+(24 * 160)); i<80; i++, pp+=2) {
@@ -103,8 +104,15 @@ void init(void)
 	aprint(LINE_CPU+3, 10, v->test_pages);
 	cprint(LINE_CPU+4, 0, "Chipset : ");
 
+	/*
+	 * Need to find out CPU type before seting up pci. This is
+	 * because AMD Opterons dont have a host bridge on dev 0.
+	 */
 	cpu_type();
 
+	/* setup pci */
+	pci_init();
+
 	/* Find the memory controller */
 	find_controller();
 
@@ -146,9 +154,10 @@ void paging_off(void)
 		"movl %%cr0, %%eax\n\t"
 		"andl $0x7FFFFFFF, %%eax\n\t"
 		"movl %%eax, %%cr0\n\t"
-		/* Disable pae */
+		/* Disable pae  & pse */
 		"movl %%cr4, %%eax\n\t"
-		"andl $0xFFFFFFDF, %%eax\n\t"
+    		"and $0xCF, %%al\n\t"
+    		"movl %%eax, %%cr4\n\t"
 		:
 		:
 		: "ax"
@@ -260,16 +269,15 @@ unsigned long page_of(void *addr)
 void cpu_type(void)
 {
 	int i, off=0;
-	int l1_cache=0, l2_cache=0;
 	ulong speed;
 
 	v->rdtsc = 0;
 	v->pae = 0;
 
 #ifdef CPUID_DEBUG
-	dprint(9,0,cpu_id.type,3,1);
-	dprint(10,0,cpu_id.model,3,1);
-	dprint(11,0,cpu_id.cpuid,3,1);
+	dprint(11,0,cpu_id.type,3,1);
+	dprint(12,0,cpu_id.model,3,1);
+	dprint(13,0,cpu_id.cpuid,3,1);
 #endif
 
 	/* If the CPUID instruction is not supported then this is */
@@ -444,32 +452,64 @@ void cpu_type(void)
 			l2_cache += cpu_id.cache_info[10];
 
 			switch(cpu_id.model) {
-			case 4:
-			case 12:
-				if (l2_cache == 256) {
-				    cprint(LINE_CPU, 0, "AMD Sempron (0.13)");
-				    off = 18;
-				} else {
-				    cprint(LINE_CPU, 0, "Athlon 64 (0.13)");
-				    off = 16;
-				}
+			default:
+				cprint(LINE_CPU, 0, "AMD K8");
+				off = 6;
 				break;
+			case 1:
 			case 5:
-				cprint(LINE_CPU, 0, "AMD Opteron (0.13)");
+				if (((cpu_id.ext >> 16) & 0xF) != 0) {
+					cprint(LINE_CPU, 0, "AMD Opteron (0.09)");				
+				} else {
+					cprint(LINE_CPU, 0, "AMD Opteron (0.13)");
+				}
 				off = 18;
 				break;
-			default:
-				cprint(LINE_CPU, 0, "AMD Athlon 64");
-				off = 13;
+			case 3:
+			case 11:
+				cprint(LINE_CPU, 0, "Athlon 64 X2");
+				off = 12;				
+				break;
+			case 4:
+			case 7:
+			case 9:
+			case 12:
+			case 14:
+			case 15:
+				if (((cpu_id.ext >> 16) & 0xF) != 0) {
+					if (l2_cache > 256) {
+						cprint(LINE_CPU, 0, "Athlon 64 (0.09)");
+					} else {
+						cprint(LINE_CPU, 0, "Sempron (0.09)");						
+					}
+				} else {
+					if (l2_cache > 256) {
+						cprint(LINE_CPU, 0, "Athlon 64 (0.13)");
+					} else {
+						cprint(LINE_CPU, 0, "Sempron (0.13)");						
+					}		
+				}
+				off = 16;
 				break;
 			}
 			break;
 		}
 		break;
 
-	/* Intel Processors */
+	/* Intel or Transmeta Processors */
 	case 'G':
-		if (cpu_id.type == 4) {
+		if ( cpu_id.vend_id[7] == 'T' ) {	/* GenuineTMx86 */
+			if (cpu_id.type == 5) {
+				cprint(LINE_CPU, 0, "TM 5x00");
+				off = 7;
+			} else if (cpu_id.type == 15) {
+				cprint(LINE_CPU, 0, "TM 8x00");
+				off = 7;
+			}
+			l1_cache = cpu_id.cache_info[3] + cpu_id.cache_info[7];
+			l2_cache = (cpu_id.cache_info[11]*256) + cpu_id.cache_info[10];
+		} else {				/* GenuineIntel */
+			if (cpu_id.type == 4) {
 			switch(cpu_id.model) {
 			case 0:
 			case 1:
@@ -646,7 +686,7 @@ void cpu_type(void)
 				off = 16;
 				break;
 			case 13:
-				if (l2_cache == 512) {
+				if (l2_cache == 1024) {
 					cprint(LINE_CPU, 0, "Celeron M (0.09)");
 				} else {
 					cprint(LINE_CPU, 0, "Pentium M (0.09)");
@@ -689,7 +729,6 @@ void cpu_type(void)
 				}
 				break;
 			case 3:
-			case 4:
 				if (l2_cache == 256) {
 					cprint(LINE_CPU, 0, "Celeron (0.09)");
 					off = 14;	
@@ -704,60 +743,85 @@ void cpu_type(void)
 					off = 16;
 				}
 				break;
-			case 5:
-				cprint(LINE_CPU, 0, "Pentium 4");
-				off = 9;
-				break;
+ 			case 4:
+				cprint(LINE_CPU, 0, "Pentium D (0.09)");
+   				off = 16;		
+ 				break;
+			default:
+				cprint(LINE_CPU, 0, "Unknown Intel");
+				off = 13;	
+ 				break;
 			}
 			break;
+		    }
 
 		}
 		break;
 
-	/* Cyrix Processors with CPUID */
+	/* VIA/Cyrix/Centaur Processors with CPUID */
 	case 'C':
-
-		switch(cpu_id.type) {
-		case 6: // VIA C3
-			switch(cpu_id.model) {
-			case 6:
-				cprint(LINE_CPU, 0, "Cyrix III");
-				off = 9;
+		if ( cpu_id.vend_id[1] == 'e' ) {	/* CentaurHauls */
+			switch(cpu_id.type){
+			case 5:
+				cprint(LINE_CPU, 0, "Centaur 5x86");
+				off = 12;
 				break;
-			case 7:
+			case 6: // VIA C3
 				if (cpu_id.step < 8) {
-				    cprint(LINE_CPU, 0, "VIA C3 Samuel2");
-				    off = 14;
+					cprint(LINE_CPU, 0, "VIA C3 Samuel2");
+					off = 14;
 				} else {
-				    cprint(LINE_CPU, 0, "VIA C3 Ezra-T");
-				    off = 13;				
-				}	
+					cprint(LINE_CPU, 0, "Via C3 Eden");
+					off = 11;
+				}
 				break;
-			case 8:
-				cprint(LINE_CPU, 0, "VIA C3 Ezra-T");
-				off = 13;
-				break;
-			case 9:
-				cprint(LINE_CPU, 0, "VIA C3 Nehemiah");
-				off = 15;
-			break;
 			}
-			// L1 = L2 = 64 KB from Cyrix III to Nehemiah
-			l1_cache = 64;
-			l2_cache = 64;
-			break;
-		case 5:
-			switch(cpu_id.model) {
-			case 0:
-				cprint(LINE_CPU, 0, "Cyrix 6x86MX/MII");
-				off = 16;
-				break;
-			case 4:
-				cprint(LINE_CPU, 0, "Cyrix GXm");
-				off = 9;
+			l1_cache = cpu_id.cache_info[3] + cpu_id.cache_info[7];
+			l2_cache = cpu_id.cache_info[11];
+		} else {				/* CyrixInstead */
+			switch(cpu_id.type) {
+			case 5:
+				switch(cpu_id.model) {
+				case 0:
+					cprint(LINE_CPU, 0, "Cyrix 6x86MX/MII");
+					off = 16;
+					break;
+				case 4:
+					cprint(LINE_CPU, 0, "Cyrix GXm");
+					off = 9;
+					break;
+				}
+				return;
+
+			case 6: // VIA C3
+				switch(cpu_id.model) {
+				case 6:
+					cprint(LINE_CPU, 0, "Cyrix III");
+					off = 9;
+					break;
+				case 7:
+					if (cpu_id.step < 8) {
+						cprint(LINE_CPU, 0, "VIA C3 Samuel2");
+						off = 14;
+					} else {
+						cprint(LINE_CPU, 0, "VIA C3 Ezra-T");
+						off = 13;
+					}
+					break;
+				case 8:
+					cprint(LINE_CPU, 0, "VIA C3 Ezra-T");
+					off = 13;
+					break;
+				case 9:
+					cprint(LINE_CPU, 0, "VIA C3 Nehemiah");
+					off = 15;
+					break;
+				}
+				// L1 = L2 = 64 KB from Cyrix III to Nehemiah
+				l1_cache = 64;
+				l2_cache = 64;
 				break;
 			}
-			return;
 		}
 		break;
 
@@ -796,12 +860,12 @@ void cpu_type(void)
 	/* To measure L1 cache speed we use a block size that is 1/4th */
 	/* of the total L1 cache size since half of it is for instructions */
 	if (l1_cache) {
-		cprint(LINE_CPU+1, 10, "     K     ");
+		cprint(LINE_CPU+1, 0, "L1 Cache:     K  ");
 		dprint(LINE_CPU+1, 11, l1_cache, 3, 0);
 		if ((speed=memspeed((ulong)mapping(0x100),
-				(l1_cache / 4) * 1024, 50))) {
-			cprint(LINE_CPU+1, 15, "      MB/s");
-			dprint(LINE_CPU+1, 15, speed, 6, 0);
+				(l1_cache / 4) * 1024, 50, MS_COPY))) {
+			cprint(LINE_CPU+1, 16, "       MB/s");
+			dprint(LINE_CPU+1, 16, speed, 6, 0);
 		}
 	}
 
@@ -810,8 +874,7 @@ void cpu_type(void)
 	/* the size of the L1 cache.  We have to fudge if the L1 */
 	/* cache is bigger than the L2 */
 	if (l2_cache) {
-		cprint(LINE_CPU+2, 9, "     K     ");
-		cprint(LINE_CPU+2, 0, "L2 Cache:    ?K");
+		cprint(LINE_CPU+2, 0, "L2 Cache:     K  ");
 		dprint(LINE_CPU+2, 10, l2_cache, 4, 0);
 
 		if (l2_cache < l1_cache) {
@@ -819,13 +882,14 @@ void cpu_type(void)
 		} else {
 			i = l1_cache;
 		}
-		if ((speed=memspeed((ulong)mapping(0x100), i*1024, 50))) {
-			cprint(LINE_CPU+2, 15, "      MB/s");
-			dprint(LINE_CPU+2, 15, speed, 6, 0);
+		if ((speed=memspeed((ulong)mapping(0x100), i*1024, 50,
+				MS_COPY))) {
+			cprint(LINE_CPU+2, 16, "       MB/s");
+			dprint(LINE_CPU+2, 16, speed, 6, 0);
 		}
 	}
 
-	/* Determine memory speed.  To find the memory spped we use */
+	/* Determine memory speed.  To find the memory speed we use */
 	/* A block size that is 5x the sum of the L1 and L2 caches */
 	i = (l2_cache + l1_cache) * 5;
 
@@ -833,9 +897,9 @@ void cpu_type(void)
 	if ((1 + (i * 2)) > (v->plim_upper << 2)) {
 		i = ((v->plim_upper <<2) - 1) / 2;
 	}
-	if((speed = memspeed((ulong)mapping(0x100), i*1024, 40))) {
-		cprint(LINE_CPU+3, 15, "      MB/s");
-		dprint(LINE_CPU+3, 15, speed, 6, 0);
+	if((speed = memspeed((ulong)mapping(0x100), i*1024, 40, MS_COPY))) {
+		cprint(LINE_CPU+3, 16, "       MB/s");
+		dprint(LINE_CPU+3, 16, speed, 6, 0);
 	}
 
 	/* Record the sarting time */
@@ -843,6 +907,8 @@ void cpu_type(void)
         v->snapl = v->startl;
         v->snaph = v->starth;
 	v->rdtsc = 1;
+	if (l1_cache == 0) { l1_cache = 66; }
+	if (l2_cache == 0) { l1_cache = 666; }
 }
 
 /* Find cache-able memory size */
@@ -871,7 +937,7 @@ static void cacheable(void)
 		}
 		/* Map the range and perform the test */
 		map_page(paddr);
-		speed = memspeed((ulong)mapping(paddr), 32*4096, 1);
+		speed = memspeed((ulong)mapping(paddr), 32*4096, 1, MS_COPY);
 		if (pspeed) {
 			if (speed < pspeed) {
 				cached -= 32;
@@ -888,7 +954,7 @@ static void cacheable(void)
 
 /* #define TICKS 5 * 11832 (count = 6376)*/
 /* #define TICKS (65536 - 12752) */
-#define TICKS (65536 - 8271)
+#define TICKS 59659	/* 50 ms */
 
 /* Returns CPU clock in khz */
 static int cpuspeed(void)
@@ -911,7 +977,7 @@ static int cpuspeed(void)
 	asm __volatile__ (
 		"rdtsc\n\t" \
 		"subl st_low,%%eax\n\t" \
-		"subl st_high,%%edx\n\t" \
+		"sbbl st_high,%%edx\n\t" \
 		:"=a" (end_low), "=d" (end_high)
 	);
 
@@ -919,13 +985,13 @@ static int cpuspeed(void)
 	if (loops < 4 || end_low < 50000) {
 		return(-1);
 	}
-	v->clks_msec = end_low/48;
+	v->clks_msec = end_low/50;
 	return(v->clks_msec);
 }
 
 /* Measure cache/memory speed by copying a block of memory. */
 /* Returned value is kbytes/second */
-static ulong memspeed(ulong src, ulong len, int iter)
+ulong memspeed(ulong src, ulong len, int iter, int type)
 {
 	ulong dst;
 	ulong wlen;
@@ -959,33 +1025,78 @@ static ulong memspeed(ulong src, ulong len, int iter)
 		"0" (cal_low), "1" (cal_high)
 	);
 
-	/* Do the first copy to prime the cache */
-	asm __volatile__ (
-		"movl %0,%%esi\n\t" \
-		"movl %1,%%edi\n\t" \
-       	 	"movl %2,%%ecx\n\t" \
-       	 	"cld\n\t" \
-       	 	"rep\n\t" \
-       	 	"movsl\n\t" \
-			:: "g" (src), "g" (dst), "g" (wlen)
-		: "esi", "edi", "ecx"
-	);
 
 	/* Now measure the speed */
-	asm __volatile__ ("rdtsc":"=a" (st_low),"=d" (st_high));
-	for (i=0; i<iter; i++) {
-	        asm __volatile__ (
+	switch (type) {
+	case MS_COPY:
+		/* Do the first copy to prime the cache */
+		asm __volatile__ (
 			"movl %0,%%esi\n\t" \
-       	 		"movl %1,%%edi\n\t" \
-       	 		"movl %2,%%ecx\n\t" \
-       	 		"cld\n\t" \
-       	 		"rep\n\t" \
-       	 		"movsl\n\t" \
-				:: "g" (src), "g" (dst), "g" (wlen)
+			"movl %1,%%edi\n\t" \
+       		 	"movl %2,%%ecx\n\t" \
+       		 	"cld\n\t" \
+       		 	"rep\n\t" \
+       		 	"movsl\n\t" \
+			:: "g" (src), "g" (dst), "g" (wlen)
 			: "esi", "edi", "ecx"
 		);
+		asm __volatile__ ("rdtsc":"=a" (st_low),"=d" (st_high));
+		for (i=0; i<iter; i++) {
+		        asm __volatile__ (
+				"movl %0,%%esi\n\t" \
+				"movl %1,%%edi\n\t" \
+       			 	"movl %2,%%ecx\n\t" \
+       			 	"cld\n\t" \
+       			 	"rep\n\t" \
+       			 	"movsl\n\t" \
+				:: "g" (src), "g" (dst), "g" (wlen)
+				: "esi", "edi", "ecx"
+			);
+		}
+		asm __volatile__ ("rdtsc":"=a" (end_low),"=d" (end_high));
+		break;
+	case MS_WRITE:
+		asm __volatile__ ("rdtsc":"=a" (st_low),"=d" (st_high));
+		for (i=0; i<iter; i++) {
+			asm __volatile__ (
+       			 	"movl %0,%%ecx\n\t" \
+				"movl %1,%%edi\n\t" \
+       			 	"movl %2,%%eax\n\t" \
+                                "rep\n\t" \
+                                "stosl\n\t"
+                                :: "g" (wlen), "g" (dst), "g" (0)
+				: "edi", "ecx", "eax"
+                        );
+		}
+		asm __volatile__ ("rdtsc":"=a" (end_low),"=d" (end_high));
+		break;
+	case MS_READ:
+		asm __volatile__ (
+			"movl %0,%%esi\n\t" \
+			"movl %1,%%ecx\n\t" \
+       	 		"cld\n\t" \
+       	 		"L1:\n\t" \
+			"lodsl\n\t" \
+       	 		"loop L1\n\t" \
+			:: "g" (src), "g" (wlen)
+			: "esi", "ecx"
+		);
+		asm __volatile__ ("rdtsc":"=a" (st_low),"=d" (st_high));
+		for (i=0; i<iter; i++) {
+		        asm __volatile__ (
+				"movl %0,%%esi\n\t" \
+				"movl %1,%%ecx\n\t" \
+       		 		"cld\n\t" \
+       		 		"L2:\n\t" \
+				"lodsl\n\t" \
+       		 		"loop L2\n\t" \
+				:: "g" (src), "g" (wlen)
+				: "esi", "ecx"
+			);
+		}
+		asm __volatile__ ("rdtsc":"=a" (end_low),"=d" (end_high));
+		break;
 	}
-	asm __volatile__ ("rdtsc":"=a" (end_low),"=d" (end_high));
 
 	/* Compute the elapsed time */
 	asm __volatile__ (
@@ -1009,8 +1120,10 @@ static ulong memspeed(ulong src, ulong len, int iter)
 		return(0);
 	}
 
-	/* Since a copy does both a read & write we need to adjuect the time */
-	end_low /= 2;
+	/* If this was a copy adjust the time */
+	if (type == MS_COPY) {
+		end_low /= 2;
+	}
 
 	/* Convert to clocks/KB */
 	end_low /= len;
