@@ -1,4 +1,4 @@
-/* lib.c - MemTest-86  Version 3.2
+/* lib.c - MemTest-86  Version 3.3
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady
@@ -311,7 +311,7 @@ void xprint(int y,int x, ulong val)
 void inter(struct eregs *trap_regs)
 {
 	int i, line;
-	char *pp;
+	unsigned char *pp;
 	ulong address = 0;
 
 	/* Get the page fault address */
