@@ -1,4 +1,4 @@
-/* main.c - MemTest-86  Version 3.2
+/* main.c - MemTest-86  Version 3.3
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady
@@ -19,7 +19,7 @@ const struct tseq tseq[] = {
 	{1, 7, 64, 66, 0,  "[Block move, 64 moves]                "},
 	{1, 2, 2, 320, 0,  "[Moving inversions, 32 bit pattern]   "},
 	{1, 9, 40, 120, 0, "[Random number sequence]              "},
-	{1, 3, 4, 240, 0,  "[Modulo 20, ones & zeros]             "},
+        {1, 11, 6, 2880, 0,"[Modulo 20, Random pattern]           "},
 	{1, 8, 1, 2, 0,    "[Bit fade test, 90 min, 2 patterns]   "},
 	{0, 0, 0, 0, 0, NULL}
 };
@@ -164,6 +164,14 @@ void do_test(void)
 		goto skip_window;
 	}
 
+	if ((ulong)&_start > LOW_TEST_ADR) {
+		/* Relocated so we need to test all selected lower memory */
+		v->map[0].start = mapping(v->plim_lower);
+		cprint(LINE_RANGE, COL_MID+28, " Relocated");
+	} else {
+		cprint(LINE_RANGE, COL_MID+28, "          ");
+	}
+
 	/* Update display of memory segments being tested */
 	lo = page_of(v->map[0].start);
 	hi = page_of(v->map[segs -1].end);
@@ -171,8 +179,6 @@ void do_test(void)
 	cprint(LINE_RANGE, COL_MID+14, " - ");
 	aprint(LINE_RANGE, COL_MID+17, hi);
 	aprint(LINE_RANGE, COL_MID+23, v->selected_pages);
-	cprint(LINE_RANGE, COL_MID+28, 
-		((ulong)&_start == LOW_TEST_ADR)?"          ":" Relocated");
 	
 #ifdef TEST_TIMES
 	{
@@ -265,7 +271,7 @@ void do_test(void)
 		}
 		break;
 
-	case 3: /* Modulo X check, all ones and zeros */
+	case 3: /* Modulo 20 check, all ones and zeros */
 		p1=0;
 		for (i=0; i<MOD_SZ; i++) {
 			p2 = ~p1;
@@ -280,7 +286,7 @@ void do_test(void)
 		}
 		break;
 
-	case 4: /* Modulo X check, 8 bit pattern */
+	case 4: /* Modulo 20 check, 8 bit pattern */
 		p0 = 0x80;
 		for (j=0; j<8; j++, p0=p0>>1) {
 			p1 = p0 | (p0<<8) | (p0<<16) | (p0<<24);
@@ -331,6 +337,23 @@ void do_test(void)
 			BAILOUT;
 		}
 		break;
+
+	case 11: /* Modulo 20 check, Random pattern */
+		for (j=0; j<tseq[v->test].iter; j++) {
+			p1 = rand();
+			for (i=0; i<MOD_SZ; i++) {
+				p2 = ~p1;
+				modtst(i, tseq[v->test].iter, p1, p2);
+				BAILOUT
+
+				/* Switch patterns */
+				p2 = p1;
+				p1 = ~p2;
+				modtst(i, tseq[v->test].iter, p1, p2);
+				BAILOUT
+			}
+		}
+		break;
 	}
  skip_window:
 	if (bail) {
@@ -367,8 +390,7 @@ void do_test(void)
 			/* We don't need the high copy for this test */
 			run_at(LOW_TEST_ADR);
 		}
-	}
-	else {
+	} else {
 		/* We have run this test in all of the windows
 		 * advance to the next test.
 		 */
@@ -392,6 +414,10 @@ void do_test(void)
 			v->pptr = 0;
 			cprint(0, COL_MID+8,
 				"                                         ");
+			if (v->ecount == 0 && v->testsel < 0) {
+			    cprint(LINE_MSG, COL_MSG,
+				"Pass complete, no errors, press Esc to exit");
+			}
 		}
 		
 		/* We always start a pass with the low copy */
