@@ -1,16 +1,23 @@
-/* pci.c - MemTest-86  Version 3.2
+/* pci.c - MemTest-86  Version 3.3
  *
  * Released under version 2 of the Gnu Public License.
- * By Eric Biederman
+ * By Chris Brady
+ * ----------------------------------------------------
+ * MemTest86+ V1.51 Specific code (GPL V2.0)
+ * By Samuel DEMEULEMEESTER, sdemeule@memtest.org
+ * http://www.x86-secret.com - http://www.memtest.org
  */
 
 #include "io.h"
 #include "pci.h"
+#include "test.h"
 
 #define PCI_CONF_TYPE_NONE 0
 #define PCI_CONF_TYPE_1    1
 #define PCI_CONF_TYPE_2    2
 
+extern struct cpu_ident cpu_id;
+
 static unsigned char pci_conf_type = PCI_CONF_TYPE_NONE;
 
 #define PCI_CONF1_ADDRESS(bus, dev, fn, reg) \
@@ -38,7 +45,7 @@ int pci_conf_read(unsigned bus, unsigned dev, unsigned fn, unsigned reg, unsigne
 	case PCI_CONF_TYPE_2:
 		outb(0xF0 | (fn << 1), 0xCF8);
 		outb(bus, 0xCFA);
-		
+
 		switch(len) {
 		case 1:  *value = inb(PCI_CONF2_ADDRESS(dev, reg)); result = 0; break;
 		case 2:  *value = inw(PCI_CONF2_ADDRESS(dev, reg)); result = 0; break;
@@ -70,7 +77,7 @@ int pci_conf_write(unsigned bus, unsigned dev, unsigned fn, unsigned reg, unsign
 	case PCI_CONF_TYPE_2:
 		outb(0xF0 | (fn << 1), 0xCF8);
 		outb(bus, 0xCFA);
-		
+
 		switch(len) {
 		case 1: outb(value, PCI_CONF2_ADDRESS(dev, reg)); result = 0; break;
 		case 2: outw(value, PCI_CONF2_ADDRESS(dev, reg)); result = 0; break;
@@ -103,31 +110,43 @@ static int pci_sanity_check(void)
 
 static int pci_check_direct(void)
 {
-	unsigned int tmp;
-
-	/* Check if configuration type 1 works. */
-	pci_conf_type = PCI_CONF_TYPE_1;
-	outb(0x01, 0xCFB);
-	tmp = inl(0xCF8);
-	outl(0x80000000, 0xCF8);
-	if ((inl(0xCF8) == 0x80000000) && (pci_sanity_check() == 0)) {
-		outl(tmp, 0xCF8);
-		return 0;
+	unsigned char tmpCFB;
+	unsigned int  tmpCF8;
+	
+	if (cpu_id.vend_id[0] == 'A' && cpu_id.type == 15) {
+			pci_conf_type = PCI_CONF_TYPE_1;
+			return 0;
+	} else {
+			/* Check if configuration type 1 works. */
+			pci_conf_type = PCI_CONF_TYPE_1;
+			tmpCFB = inb(0xCFB);
+			outb(0x01, 0xCFB);
+			tmpCF8 = inl(0xCF8);
+			outl(0x80000000, 0xCF8);
+			if ((inl(0xCF8) == 0x80000000) && (pci_sanity_check() == 0)) {
+				outl(tmpCF8, 0xCF8);
+				outb(tmpCFB, 0xCFB);
+				return 0;
+			}
+			outl(tmpCF8, 0xCF8);
+		
+			/* Check if configuration type 2 works. */
+			pci_conf_type = PCI_CONF_TYPE_2;
+			outb(0x00, 0xCFB);
+			outb(0x00, 0xCF8);
+			outb(0x00, 0xCFA);
+			if (inb(0xCF8) == 0x00 && inb(0xCFA) == 0x00 && (pci_sanity_check() == 0)) {
+				outb(tmpCFB, 0xCFB);
+				return 0;
 	}
-	outl(tmp, 0xCF8);
 
-	/* Check if configuration type 2 works. */
-	pci_conf_type = PCI_CONF_TYPE_2;
-	outb(0x00, 0xCFB);
-	outb(0x00, 0xCF8);
-	outb(0x00, 0xCFA);
-	if (inb(0xCF8) == 0x00 && inb(0xCFA) == 0x00 && (pci_sanity_check() == 0)) {
-		return 0;
-	}
+	outb(tmpCFB, 0xCFB);
 
 	/* Nothing worked return an error */
 	pci_conf_type = PCI_CONF_TYPE_NONE;
 	return -1;
+	
+	}
 }
 
 int pci_init(void)
