@@ -1,4 +1,4 @@
-/* test.c - MemTest-86  Version 3.2
+/* test.c - MemTest-86  Version 3.3
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady
@@ -241,6 +241,162 @@ void addr_tst2()
 	}
 }
 
+/*
+ * Test all of memory using a "half moving inversions" algorithm using random
+ * numbers and their complment as the data pattern. Since we are not able to
+ * produce random numbers in reverse order testing is only done in the forward
+ * direction.
+ */
+void movinvr()
+{
+	int i, j, done, seed1, seed2;
+	volatile ulong *pe;
+	volatile ulong *start,*end;
+	ulong num;
+
+	/* Initialize memory with initial sequence of random numbers.  */
+	if (v->rdtsc) {
+		asm __volatile__ ("rdtsc":"=a" (seed1),"=d" (seed2));
+	} else {
+		seed1 = 521288629 + v->pass;
+		seed2 = 362436069 - v->pass;
+	}
+
+	/* Display the current seed */
+        hprint(LINE_PAT, COL_PAT, seed1);
+	rand_seed(seed1, seed2);
+	for (j=0; j<segs; j++) {
+		start = v->map[j].start;
+		end = v->map[j].end;
+		pe = start;
+		p = start;
+		done = 0;
+		do {
+			/* Check for overflow */
+			if (pe + SPINSZ > pe) {
+				pe += SPINSZ;
+			} else {
+				pe = end;
+			}
+			if (pe >= end) {
+				pe = end;
+				done++;
+			}
+			if (p == pe ) {
+				break;
+			}
+/* Original C code replaced with hand tuned assembly code */
+/*
+			for (; p < pe; p++) {
+				*p = rand();
+			}
+ */
+
+                        asm __volatile__ (
+                                "jmp L200\n\t"
+                                ".p2align 4,,7\n\t"
+                                "L200:\n\t"
+                                "call rand\n\t"
+				"movl %%eax,(%%edi)\n\t"
+                                "addl $4,%%edi\n\t"
+                                "cmpl %%ebx,%%edi\n\t"
+                                "jb L200\n\t"
+                                : "=D" (p)
+                                : "D" (p), "b" (pe)
+				: "eax"
+                        );
+
+			do_tick();
+			BAILR
+		} while (!done);
+	}
+
+	/* Do moving inversions test. Check for initial pattern and then
+	 * write the complement for each memory location. Test from bottom
+	 * up and then from the top down.  */
+	for (i=0; i<2; i++) {
+		rand_seed(seed1, seed2);
+		for (j=0; j<segs; j++) {
+			start = v->map[j].start;
+			end = v->map[j].end;
+			pe = start;
+			p = start;
+			done = 0;
+			do {
+				/* Check for overflow */
+				if (pe + SPINSZ > pe) {
+					pe += SPINSZ;
+				} else {
+					pe = end;
+				}
+				if (pe >= end) {
+					pe = end;
+					done++;
+				}
+				if (p == pe ) {
+					break;
+				}
+/* Original C code replaced with hand tuned assembly code */
+/*
+				for (; p < pe; p++) {
+					num = rand();
+					if (i) {
+						num = ~num;
+					}
+					if ((bad=*p) != num) {
+						error((ulong*)p, num, bad);
+					}
+					*p = ~num;
+				}
+*/
+				if (i) {
+					num = 0xffffffff;
+				} else {
+					num = 0;
+				}
+				asm __volatile__ (
+					"jmp L26\n\t" \
+
+					".p2align 4,,7\n\t" \
+					"L26:\n\t" \
+					"call rand\n\t"
+					"xorl %%ebx,%%eax\n\t" \
+					"movl (%%edi),%%ecx\n\t" \
+					"cmpl %%eax,%%ecx\n\t" \
+					"jne L23\n\t" \
+					"L25:\n\t" \
+					"movl $0xffffffff,%%edx\n\t" \
+					"xorl %%edx,%%eax\n\t" \
+					"movl %%eax,(%%edi)\n\t" \
+					"addl $4,%%edi\n\t" \
+					"cmpl %%esi,%%edi\n\t" \
+					"jb L26\n\t" \
+					"jmp L24\n" \
+
+					"L23:\n\t" \
+					"pushl %%esi\n\t" \
+					"pushl %%ecx\n\t" \
+					"pushl %%eax\n\t" \
+					"pushl %%edi\n\t" \
+					"call error\n\t" \
+					"popl %%edi\n\t" \
+					"popl %%eax\n\t" \
+					"popl %%ecx\n\t" \
+					"popl %%esi\n\t" \
+					"jmp L25\n" \
+
+					"L24:\n\t" \
+					: "=D" (p)
+					: "D" (p), "S" (pe), "b" (num)
+					: "eax", "ecx", "edx"
+				);
+				do_tick();
+				BAILR
+			} while (!done);
+		}
+	}
+}
+
 /*
  * Test all of memory using a "moving inversions" algorithm using the
  * pattern in p1 and it's complement in p2.
@@ -434,173 +590,15 @@ void movinv1(int iter, ulong p1, ulong p2)
 	}
 }
 
-/*
- * Test all of memory using a "half moving inversions" algorithm using random
- * numbers and their complment as the data pattern. Since we are not able to
- * produce random numbers in reverse order testing is only done in the forward
- * direction.
- */
-void movinvr()
-{
-	int i, j, done, seed1, seed2;
-	volatile ulong *pe;
-	volatile ulong *start,*end;
-	ulong num;
-
-	/* Initialize memory with initial sequence of random numbers.  */
-	if (v->rdtsc) {
-		asm __volatile__ ("rdtsc":"=a" (seed1),"=d" (seed2));
-	} else {
-		seed1 = 521288629 + v->pass;
-		seed2 = 362436069 - v->pass;
-	}
-
-	/* Display the current seed */
-        hprint(LINE_PAT, COL_PAT, seed1);
-	rand_seed(seed1, seed2);
-	for (j=0; j<segs; j++) {
-		start = v->map[j].start;
-		end = v->map[j].end;
-		pe = start;
-		p = start;
-		done = 0;
-		do {
-			/* Check for overflow */
-			if (pe + SPINSZ > pe) {
-				pe += SPINSZ;
-			} else {
-				pe = end;
-			}
-			if (pe >= end) {
-				pe = end;
-				done++;
-			}
-			if (p == pe ) {
-				break;
-			}
-/* Original C code replaced with hand tuned assembly code */
-/*
-			for (; p < pe; p++) {
-				*p = rand();
-			}
- */
-
-                        asm __volatile__ (
-                                "jmp L200\n\t"
-                                ".p2align 4,,7\n\t"
-                                "L200:\n\t"
-                                "call rand\n\t"
-				"movl %%eax,(%%edi)\n\t"
-                                "addl $4,%%edi\n\t"
-                                "cmpl %%ebx,%%edi\n\t"
-                                "jb L200\n\t"
-                                : "=D" (p)
-                                : "D" (p), "b" (pe)
-				: "eax"
-                        );
-
-			do_tick();
-			BAILR
-		} while (!done);
-	}
-
-	/* Do moving inversions test. Check for initial pattern and then
-	 * write the complement for each memory location. Test from bottom
-	 * up and then from the top down.  */
-	for (i=0; i<2; i++) {
-		rand_seed(seed1, seed2);
-		for (j=0; j<segs; j++) {
-			start = v->map[j].start;
-			end = v->map[j].end;
-			pe = start;
-			p = start;
-			done = 0;
-			do {
-				/* Check for overflow */
-				if (pe + SPINSZ > pe) {
-					pe += SPINSZ;
-				} else {
-					pe = end;
-				}
-				if (pe >= end) {
-					pe = end;
-					done++;
-				}
-				if (p == pe ) {
-					break;
-				}
-/* Original C code replaced with hand tuned assembly code */
-/*
-				for (; p < pe; p++) {
-					num = rand();
-					if (i) {
-						num = ~num;
-					}
-					if ((bad=*p) != num) {
-						error((ulong*)p, num, bad);
-					}
-					*p = ~num;
-				}
-*/
-				if (i) {
-					num = 0xffffffff;
-				} else {
-					num = 0;
-				}
-				asm __volatile__ (
-					"jmp L26\n\t" \
-
-					".p2align 4,,7\n\t" \
-					"L26:\n\t" \
-					"call rand\n\t"
-					"xorl %%ebx,%%eax\n\t" \
-					"movl (%%edi),%%ecx\n\t" \
-					"cmpl %%eax,%%ecx\n\t" \
-					"jne L23\n\t" \
-					"L25:\n\t" \
-					"movl $0xffffffff,%%edx\n\t" \
-					"xorl %%edx,%%eax\n\t" \
-					"movl %%eax,(%%edi)\n\t" \
-					"addl $4,%%edi\n\t" \
-					"cmpl %%esi,%%edi\n\t" \
-					"jb L26\n\t" \
-					"jmp L24\n" \
-
-					"L23:\n\t" \
-					"pushl %%esi\n\t" \
-					"pushl %%ecx\n\t" \
-					"pushl %%eax\n\t" \
-					"pushl %%edi\n\t" \
-					"call error\n\t" \
-					"popl %%edi\n\t" \
-					"popl %%eax\n\t" \
-					"popl %%ecx\n\t" \
-					"popl %%esi\n\t" \
-					"jmp L25\n" \
-
-					"L24:\n\t" \
-					: "=D" (p)
-					: "D" (p), "S" (pe), "b" (num)
-					: "eax", "ecx", "edx"
-				);
-				do_tick();
-				BAILR
-			} while (!done);
-		}
-	}
-}
-
 void movinv32(int iter, ulong p1, ulong lb, ulong hb, int sval, int off)
 {
-	int i, j, k=0, n = 0, done;
+	int i, j, k=0, done;
 	volatile ulong *pe;
 	volatile ulong *start, *end;
-	ulong pat, p3;
-
-	p3 = sval << 31;
+	ulong pat = 0;
 
 	/* Display the current pattern */
-        hprint(LINE_PAT, COL_PAT, p1);
+	hprint(LINE_PAT, COL_PAT, p1);
 
 	/* Initialize memory with the initial pattern.  */
 	for (j=0; j<segs; j++) {
@@ -643,24 +641,18 @@ void movinv32(int iter, ulong p1, ulong lb, ulong hb, int sval, int off)
 				"jmp L20\n\t"
 				".p2align 4,,7\n\t"
 
+/* CDH start */
 				"L20:\n\t"
 				"movl %%ecx,(%%edi)\n\t"
-				"addl $1,%%ebx\n\t"
-				"cmpl $32,%%ebx\n\t"
-				"jne L21\n\t"
-				"movl %%esi,%%ecx\n\t"
-				"xorl %%ebx,%%ebx\n\t"
-				"jmp L22\n"
-				"L21:\n\t"
-				"shll $1,%%ecx\n\t"
-				"orl %%eax,%%ecx\n\t"
-				"L22:\n\t"
+				"incb %%bl\n\t"
 				"addl $4,%%edi\n\t"
+				"roll $1,%%ecx\n\t"
 				"cmpl %%edx,%%edi\n\t"
 				"jb L20\n\t"
-				: "=b" (k), "=D" (p)
-				: "D" (p),"d" (pe),"b" (k),"c" (pat),
-					"a" (sval), "S" (lb)
+				"andb $31,%%bl\n\t"
+				: "=b" (k), "=D" (p), "=c" (pat)
+				: "D" (p),"d" (pe),"b" (k),"c" (pat)
+/* CDH end */
 			);
 			do_tick();
 			BAILR
@@ -719,24 +711,18 @@ void movinv32(int iter, ulong p1, ulong lb, ulong hb, int sval, int off)
 					"cmpl %%ecx,%%ebp\n\t"
 					"jne L34\n\t"
 
+/* CDH start */
 					"L35:\n\t"
 					"notl %%ecx\n\t"
 					"movl %%ecx,(%%edi)\n\t"
 					"notl %%ecx\n\t"
-					"incl %%ebx\n\t"
-					"cmpl $32,%%ebx\n\t"
-					"jne L31\n\t"
-					"movl %%esi,%%ecx\n\t"
-					"xorl %%ebx,%%ebx\n\t"
-					"jmp L32\n"
-					"L31:\n\t"
-					"shll $1,%%ecx\n\t"
-					"orl %%eax,%%ecx\n\t"
-					"L32:\n\t"
 					"addl $4,%%edi\n\t"
+					"incb %%bl\n\t"
+					"roll $1,%%ecx\n\t"
 					"cmpl %%edx,%%edi\n\t"
 					"jb L30\n\t"
 					"jmp L33\n\t"
+/* CDH end */
 
 					"L34:\n\t" \
 					"pushl %%esi\n\t"
@@ -756,11 +742,13 @@ void movinv32(int iter, ulong p1, ulong lb, ulong hb, int sval, int off)
 					"popl %%esi\n\t"
 					"jmp L35\n"
 
+/* CDH start */
 					"L33:\n\t"
+					"andb $31,%%bl\n\t"
 					"popl %%ebp\n\t"
-					: "=b" (k), "=D" (p)
-					: "D" (p),"d" (pe),"b" (k),"c" (pat),
-						"a" (sval), "S" (lb)
+					: "=b" (k), "=D" (p), "=c" (pat)
+					: "D" (p),"d" (pe),"b" (k),"c" (pat)
+/* CDH end */
 				);
 				do_tick();
 				BAILR
@@ -770,14 +758,26 @@ void movinv32(int iter, ulong p1, ulong lb, ulong hb, int sval, int off)
 		/* Since we already adjusted k and the pattern this
 		 * code backs both up one step
 		 */
-		if (--k < 0) {
-			k = 31;
-		}
-		for (pat = lb, n = 0; n < k; n++) {
-			pat = pat << 1;
-			pat |= sval;
-		}
-		k++;
+/* CDH start */
+/* Original C code replaced with hand tuned assembly code
+ *		pat = lb;
+ *		if ( 0 != (k = (k-1) & 31) ) {
+ *			pat = (pat << k);
+ *			if ( sval )
+ *			pat |= ((sval << k) - 1);
+ *		}
+ *		k++;
+ */
+			asm __volatile__ (
+			"decl %%ecx\n\t"
+			"andl $31,%%ecx\n\t"
+			"roll %%cl,%%ebx\n\t"
+			"incb %%cl\n\t"
+			: "=c" (k), "=b" (pat)
+			: "c" (k), "b" (lb)
+			);
+/* CDH end */
+
 		for (j=segs-1; j>=0; j--) {
 			start = v->map[j].start;
 			end = v->map[j].end;
@@ -817,7 +817,7 @@ void movinv32(int iter, ulong p1, ulong lb, ulong hb, int sval, int off)
 					"pushl %%ebp\n\t"
 					"addl $4,%%edi\n\t"
 					"jmp L40\n\t"
-	
+
 					".p2align 4,,7\n\t"
 					"L40:\n\t"
 					"subl $4,%%edi\n\t"
@@ -826,22 +826,16 @@ void movinv32(int iter, ulong p1, ulong lb, ulong hb, int sval, int off)
 					"cmpl %%ecx,%%ebp\n\t"
 					"jne L44\n\t"
 
+/* CDH start */
 					"L45:\n\t"
 					"notl %%ecx\n\t"
 					"movl %%ecx,(%%edi)\n\t"
-					"decl %%ebx\n\t"
-					"cmpl $0,%%ebx\n\t"
-					"jg L41\n\t"
-					"movl %%esi,%%ecx\n\t"
-					"movl $32,%%ebx\n\t"
-					"jmp L42\n"
-					"L41:\n\t"
-					"shrl $1,%%ecx\n\t"
-					"orl %%eax,%%ecx\n\t"
-					"L42:\n\t"
+					"decb %%bl\n\t"
+					"rorl $1,%%ecx\n\t"
 					"cmpl %%edx,%%edi\n\t"
 					"ja L40\n\t"
 					"jmp L43\n\t"
+/* CDH end */
 
 					"L44:\n\t" \
 					"pushl %%esi\n\t"
@@ -861,12 +855,14 @@ void movinv32(int iter, ulong p1, ulong lb, ulong hb, int sval, int off)
 					"popl %%esi\n\t"
 					"jmp L45\n"
 
+/* CDH start */
 					"L43:\n\t"
+					"andb $31,%%bl\n\t"
 					"subl $4,%%edi\n\t"
 					"popl %%ebp\n\t"
 					: "=b" (k), "=D" (p), "=c" (pat)
-					: "D" (p),"d" (pe),"b" (k),"c" (pat),
-						"a" (p3), "S" (hb)
+					: "D" (p),"d" (pe),"b" (k),"c" (pat)
+/* CDH end */
 				);
 				do_tick();
 				BAILR
@@ -1359,7 +1355,7 @@ void error(ulong *adr, ulong good, ulong bad)
 		/* Don't display duplicate errors */
 		if ((ulong)adr == (ulong)v->eadr && xor == v->exor) {
 			print_err_counts();
-			dprint(v->msg_line, 62, ++ecount, 5, 0);
+			dprint(v->msg_line, 66, ++ecount, 5, 0);
 			return;
 		}
 		print_err(adr, good, bad, xor);
@@ -1439,8 +1435,13 @@ void print_hdr(void)
 
 static void update_err_counts(void)
 {
+	if (v->pass && v->ecount == 0) {
+		cprint(LINE_MSG, COL_MSG,
+			"                                            ");
+	}
 	++(v->ecount);
 	tseq[v->test].errors++;
+		
 }
 
 static void print_err_counts(void)
@@ -1448,6 +1449,9 @@ static void print_err_counts(void)
 	int i;
 	char *pp;
 
+	if ((v->ecount > 1048756) && (v->ecount % 32768 != 0)) return;
+	if ((v->ecount > 2048) && (v->ecount % 1024 != 0)) return;
+
 	dprint(LINE_INFO, COL_ERR, v->ecount, 6, 0);
 	dprint(LINE_INFO, COL_ECC_ERR, v->ecc_ecount, 6, 0);
 
