@@ -1,4 +1,4 @@
-/* test.h - MemTest-86  Version 3.2
+/* test.h - MemTest-86  Version 3.3
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady
@@ -67,6 +67,8 @@ typedef unsigned long ulong;
 #define LINE_HEADER	12
 #define LINE_SCROLL	14
 #define BAR_SIZE	(78-COL_MID-9)
+#define LINE_MSG	18
+#define COL_MSG		18
 
 #define POP_W	30
 #define POP_H	15
@@ -74,11 +76,16 @@ typedef unsigned long ulong;
 #define POP_Y	8
 #define NULL	0
 
+/* sizeof spd window */
+#define SPD_W   51      
+#define SPD_H   19      
+#define SPD_X   16
+#define SPD_Y   4       
+
 /* memspeed operations */
-#define MS_BCOPY	1
-#define MS_COPY		2
-#define MS_WRITE	3
-#define MS_READ		4
+#define MS_COPY		1
+#define MS_WRITE	2
+#define MS_READ		3
 
 #define SZ_MODE_BIOS		1
 #define SZ_MODE_BIOS_RES	2
@@ -148,9 +155,11 @@ void parity_err(ulong edi, ulong esi);
 void start_config(void);
 void clear_screen(void);
 void paging_off(void);
+void show_spd(void);
 int map_page(unsigned long page);
 void *mapping(unsigned long page_address);
 void *emapping(unsigned long page_address);
+ulong memspeed(ulong src, ulong len, int iter, int type);
 unsigned long page_of(void *ptr);
 
 #define PRINTMODE_ADDRESSES 0
@@ -238,6 +247,7 @@ struct cpu_ident {
 	char vend_id[12];
 	unsigned char cache_info[16];
 	long pwrcap;
+	long ext;
 };
 #define X86_FEATURE_PAE		(0*32+ 6) /* Physical Address Extensions */
 
